package com.yifan.yf_ads.admanager;

import android.app.Activity;
import android.content.Intent;
import android.view.ViewGroup;

import com.yfanads.android.core.splash.YFAdSplashAds;
import com.yfanads.android.core.splash.YFSplashListener;
import com.yfanads.android.model.YFAdError;
import com.yfanads.android.utils.InitUtils;
import com.yfanads.android.utils.YFLog;
import com.yifan.yf_ads.activity.SplashCallMgr;
import com.yifan.yf_ads.activity.YFSplashActivity;
import com.yifan.yf_ads.bean.SplashBean;
import com.yifan.yf_ads.utils.Constants;
import com.yifan.yf_ads.utils.FLog;

import org.json.JSONException;
import org.json.JSONObject;

import io.flutter.plugin.common.EventChannel;

/**
 * 一句话描述该类功能.
 *
 * @author JamesQian
 * @version 1.0
 * @copyright 亿帆
 * @date 2024/5/11 11:26
 **/
public class SplashAdManager extends CommonAdManager implements SplashCallMgr.AdManagerIF {

    private static final String TAG = "SplashAdManager";

    private YFAdSplashAds splashAds;

    private SplashBean splashBean;

    // 标记从跳转到落地页或拉起其它应用，当isLeftActivity为true时，忽略onSplashAdDismiss回调
    public static boolean isLeftActivity = false;
    // 标记广告是否被点击
    // 广告点击后会有两种情况：
    // 1. 跳转到落地页或者拉起其它应用,当广告展示时间完毕后回调onSplashAdDismiss，此时应该忽略，
    //    当从落地页返回到开屏广告页面时，要跳转到应用主界面
    // 2. 进入下载操作，界面还是停留在开屏广告界面直到展示完毕回调onSplashAdDismiss
    public static boolean isClicked = false;

    // 标记广告是否已关闭
    // 当开屏页在后台时，回调onSplashAdDismiss不跳转到主界面；
    // 等开屏页到前台后，跳转到主界面
    public static boolean isAdDismiss = false;

    // load 状态：0 未请求， 1正在请求 2 请求成功 3 正在展示
    private int isLoadStatue = 0;

    public SplashAdManager(Activity context) {
        super(context, Constants.AD_TYPE_SPLASH);
    }

    public void requestSplashAD(String posId, SplashBean splashBean, EventChannel.EventSink eventSink) {
        this.splashBean = splashBean;

        if (splashAds != null) {
            releaseAd();
        }
        splashAds = new YFAdSplashAds(activity, new YFSplashListener() {
            @Override
            public void onAdSuccess() {
                FLog.i("onAdSuccess ");
                isLoadStatue = 2;
                try {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onAdSuccess");
                    if (InitUtils.isEpsOpen()) {
                        jsonObject.put(Constants.KEY_ECPM, splashAds.getEcpm() + "");
                    }
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
                if (splashBean.isLoadAndShow()) {
                    showAd(activity);
                }
            }

            @Override
            public void onAdExposure() {
                FLog.i("onAdExposure ");
                isLoadStatue = 3;
                if (SplashCallMgr.getSplashAdCall() != null) {
                    SplashCallMgr.getSplashAdCall().removeHandler();
                }
                try {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "oAdShow");
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onAdClicked() {
                FLog.i("onAdClicked ");
                isClicked = true;
                try {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onAdClicked");
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }

            @Override
            public void onAdClosed() {
                isLoadStatue = 0;
                isAdDismiss = true;
                try {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onAdClosed");
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
                FLog.i("onAdClosed " + isLeftActivity);
                if (/*isClicked && */isLeftActivity) {
                    return;
                }
                FLog.i("onAdClosed " + SplashCallMgr.getSplashAdCall());
                if (SplashCallMgr.getSplashAdCall() != null) {
                    SplashCallMgr.getSplashAdCall().gotoMain();
                }
            }

            @Override
            public void onAdFailed(YFAdError yfAdError) {
                FLog.i("onAdFailed " + yfAdError.msg);
                isLoadStatue = 0;
                isAdDismiss = true;
                try {
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put(Constants.KEY_METHOD, "onFailed");
                    jsonObject.put(Constants.KEY_ERROR, yfAdError.msg);
                    callBack(jsonObject, eventSink);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
                if (/*isClicked && */isLeftActivity) {
                    return;
                }
                if (SplashCallMgr.getSplashAdCall() != null) {
                    SplashCallMgr.getSplashAdCall().gotoMain();
                    releaseAd();
                }
            }
        });
        splashAds.loadOnly(posId);
    }

    public void showAd(Activity activity) {
        YFLog.high("showAd isLoadStatue " + isLoadStatue);
        if (isLoadStatue == 2 && splashAds != null) {
            // 跳转到新页面
            SplashCallMgr.setAdManagerIF(this);
            String positionId = splashBean.getPositionId();
            String destActivityFullPath = splashBean.getDestActivityFullPath();
            String splashClassPath = splashBean.getSplashClassPath();
            Intent intent;
            if (splashClassPath != null && splashClassPath.length() > 0) {
                try {
                    Class<?> realClass = Class.forName(splashClassPath);
                    intent = new Intent(activity, realClass);
                } catch (Throwable e) {
                    intent = new Intent(activity, YFSplashActivity.class);
                }
            } else {
                intent = new Intent(activity, YFSplashActivity.class);
            }
            intent.putExtra(Constants.KEY_DEST_ACTIVITY_FULL_PATH, destActivityFullPath);
            activity.startActivity(intent);
        }
    }

    @Override
    public void showAds(Activity activity, ViewGroup adContainer) {
        if (this.activity != null && splashAds != null) {
            splashAds.showAds(this.activity, adContainer);
        }
    }

    @Override
    public boolean isLoadSuccess() {
        return isLoadStatue == 2;
    }

    @Override
    public void releaseAd() {
        if (splashAds != null) {
            splashAds.destroy();
        }
        isClicked = false;
        isAdDismiss = false;
        isLeftActivity = false;
    }

}
