package com.yifan.yf_ads.views.express;

import android.app.Activity;
import android.content.Context;
import android.graphics.Color;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewTreeObserver;
import android.widget.LinearLayout;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.yfanads.android.model.ExpView;
import com.yfanads.android.utils.ScreenUtil;
import com.yfanads.android.utils.YFUtil;
import com.yifan.yf_ads.admanager.CommonAdManager;
import com.yifan.yf_ads.admanager.ExpressAdManager;
import com.yifan.yf_ads.bean.ExpressAdBean;
import com.yifan.yf_ads.utils.Constants;
import com.yifan.yf_ads.utils.DeviceUtils;
import com.yifan.yf_ads.utils.FLog;

import org.json.JSONObject;

import java.util.logging.Handler;

import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.platform.PlatformView;

/**
 * 一句话描述该类功能.
 *
 * @author JamesQian
 * @version 1.0
 * @copyright 亿帆
 * @date 2024/1/26 10:30
 **/
public class ExpressView implements PlatformView {

    private final static String TAG = "ExpressFeedView";

    private ExpressAdManager expressAdManager;

    @NonNull
    private LinearLayout containerView;

    private boolean isShow;

    ExpressView(@NonNull final Activity context, final String id, @Nullable
    final ExpressAdBean creationParams, final EventChannel.EventSink eventSink, ExpressCall expressCall) {
        FLog.i(TAG, "NativeView id = " + id);
        if (creationParams == null) {
            return;
        }
        if (expressAdManager == null) {
            expressAdManager = new ExpressAdManager(context, Constants.AD_TYPE_EXPRESS_FEED);
        }

        containerView = new LinearLayout(context);
        int width = creationParams.getContainerWidth();
        int height = creationParams.getContainerHeight();
        int acceptWidth = width > 0 ? ScreenUtil.dip2px(context, width) : ScreenUtil.getScreenWidth(context);
        FLog.i(TAG, "requestExpressFeedAd. width: " + width + " , acceptWidth: " + acceptWidth + ",height:" + height);
        LinearLayout.LayoutParams layoutParams = new LinearLayout.LayoutParams(acceptWidth, ViewGroup.LayoutParams.WRAP_CONTENT);
        containerView.setLayoutParams(layoutParams);
        expressAdManager.setAdListener(new ExpressAdManager.YFFlutterExpressFeedListener() {
            @Override
            public void expressFeedAdLoadSuccess(ExpView view) {
                FLog.i(TAG, "expressFeedAdLoadSuccess ");
                if (view == null) {
                    return;
                }
                if (view.getView() != null) {
                    onContainerSizeChanged(view.getView(), id, eventSink);
                } else {
                    view.render(containerView);
                }
            }

            @Override
            public void onAdClose() {
                FLog.i(TAG, "onAdClose ");
                containerView.removeAllViews();
                ViewGroup.LayoutParams layoutParams = containerView.getLayoutParams();
                layoutParams.height = 0;
                containerView.setLayoutParams(layoutParams);

                if (expressCall != null) {
                    expressCall.onAdClose(id);
                }
            }

            @Override
            public void onAdLoadSuccess() {
                if (creationParams.isAutoShow()) {
                    showAd(context);
                }
            }

            @Override
            public void onAdLoadFail(String errorMessage) {
                FLog.i(TAG, "expressFeedAdLoadFail:" + errorMessage);
            }
        });
        expressAdManager.requestExpressFeedAd(id, creationParams, eventSink);
    }

    public boolean isShow() {
        return isShow;
    }

    public void showAd(Activity context) {
        if (expressAdManager != null) {
            expressAdManager.showAd(context);
            isShow = true;
        }
    }

    private void onContainerSizeChanged(View view, String viewId, final EventChannel.EventSink eventSink) {
        containerView.removeAllViews();
        containerView.addView(view);
    }

    @Override
    public View getView() {
        return containerView;
    }

    @Override
    public void dispose() {
        FLog.i(TAG, "dispose");
//        releaseAd();
    }

    public void releaseAd() {
        if (expressAdManager != null) {
            expressAdManager.destroy();
        }
    }
}
