#import "YfAdsPlugin.h"
#import "YFAdsConstant.h"
#import "YFAdsSDKManager.h"
#import "YFAdsInsertManager.h"
#import "YFAdsRewardVideoManager.h"
#import "YFAdsSplashManager.h"
#import "YFAdsUtils.h"
#import "YFAdsBannerView.h"
#import "YFAdsBannerViewFactory.h"
#import "YFAdsVideoView.h"
#import "YFAdsVideoViewFactory.h"
#import "YFAdsNativeView.h"
#import "YFAdsNativeViewFactory.h"


@interface YfAdsPlugin ()<FlutterStreamHandler>

@property (nonatomic,strong) FlutterEventChannel *flutterEventChannel;
@property (nonatomic,strong) FlutterEventSink flutterEventSink;
@property (nonatomic,strong) UIViewController *flutterViewController;
@property (nonatomic,strong) YFAdsInsertManager *insertAdManager;
@property (nonatomic,strong) YFAdsRewardVideoManager *rewardVideoManager;
@property (nonatomic,strong) YFAdsSDKManager *reaperSdkManager;
@property (nonatomic,strong) YFAdsSplashManager *splash;
@property (nonatomic,strong) YFAdsBannerViewFactory *bannerViewFactory;
@property (nonatomic,strong) YFAdsNativeViewFactory *nativeViewFactory;
@property (nonatomic,strong) YFAdsVideoViewFactory *videoPathViewFactory;
@property (nonatomic,strong) YFAdsBannerView *bannerView;


@end

@implementation YfAdsPlugin

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    FlutterMethodChannel* channel = [FlutterMethodChannel
            methodChannelWithName:@"yf_ads_plugin"
                  binaryMessenger:[registrar messenger]];
    YfAdsPlugin *instance =
            [[YfAdsPlugin alloc] initWithRegistrar:registrar];
    [registrar addMethodCallDelegate:instance channel:channel];
}

- (instancetype)initWithRegistrar:(NSObject<FlutterPluginRegistrar> *)registrar {
    self = [super init];
    if (self) {
        _flutterEventChannel = [FlutterEventChannel eventChannelWithName:@"yf_ads_plugin_event" binaryMessenger:[registrar messenger]];
        [_flutterEventChannel setStreamHandler:self];

        _bannerViewFactory = [[YFAdsBannerViewFactory alloc] initViewFactory];
        [registrar registerViewFactory:_bannerViewFactory withId:@"com.yf/banner_platform_view"];

        _nativeViewFactory = [[YFAdsNativeViewFactory alloc] initViewFactory];
        [registrar registerViewFactory:_nativeViewFactory withId:@"com.yf/native_platform_view"];

        _videoPathViewFactory = [[YFAdsVideoViewFactory alloc] initViewFactory];
        [registrar registerViewFactory:_videoPathViewFactory withId:@"com.yf/stream_ad_platform_view"];
    }
    return self;
}

- (UIViewController *)viewControllerWithWindow:(UIWindow *)window {
    UIWindow *windowToUse = window;
    if (windowToUse == nil) {
        for (UIWindow *window in [UIApplication sharedApplication].windows) {
            if (window.isKeyWindow) {
                windowToUse = window;
                break;
            }
        }
    }

    UIViewController *topController = windowToUse.rootViewController;
    while (topController.presentedViewController) {
        topController = topController.presentedViewController;
    }
    return topController;
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {

    if(!_flutterViewController){
        _flutterViewController = [self viewControllerWithWindow:nil];
        if(_bannerViewFactory){
            [_bannerViewFactory setFlutterViewController:_flutterViewController];
        }
        if(_nativeViewFactory){
            [_nativeViewFactory setFlutterViewController:_flutterViewController];
        }
        if(_videoPathViewFactory){
            [_videoPathViewFactory setFlutterViewController:_flutterViewController];
        }
    }
    NSString *method = call.method;
    NSLog(@"handleMethodCall->%@",call);
    NSLog(@"handleMethodCall->%@",call.method);
    NSLog(@"handleMethodCall-arguments->%@",call.arguments);

    NSDictionary *parameters = [YFAdsUtils jsonString2Dictionary:call.arguments];
   
    
    
    if ([method isEqualToString:KEY_METHOD_INIT_REAPER_SDK]) {
        if(!_reaperSdkManager){
            _reaperSdkManager = [[YFAdsSDKManager alloc] init];
        }
        
        [_reaperSdkManager initReaperSdk:parameters result:result];
        NSDictionary *resultDic = @{KEY_RESULT:@"init success"};
        result([YFAdsUtils dictionary2JsonString:resultDic]);
    }else if ([method isEqualToString:KEY_METHOD_SPLASH_AD]) {
    
        _splash = [[YFAdsSplashManager alloc] initWithParams:_flutterEventSink viewController:_flutterViewController result:result];
        if(parameters[@"loadAndShow"] && ![parameters[@"loadAndShow"] isKindOfClass:NSNull.class]) {
            NSInteger isLoadAndShow = [parameters[@"loadAndShow"] integerValue];
            if (isLoadAndShow == 1) {
                [_splash loadAndShow:parameters];
            }else {
                [_splash loadAd:parameters];
            }
        }else {
            [_splash loadAndShow:parameters];
        }
        
    } else if([method isEqualToString:@"showSplashAd"]){
        if(_splash){
            [_splash showAd];
        }
    } else if ([method isEqualToString:KEY_METHOD_REQUEST_INTERACTION_AD]) {
    
        _insertAdManager = [[YFAdsInsertManager alloc] initWithParams:_flutterEventSink viewController:_flutterViewController result:result];
        if(parameters[@"loadAndShow"] && ![parameters[@"loadAndShow"] isKindOfClass:NSNull.class]) {
            NSInteger isLoadAndShow = [parameters[@"loadAndShow"] integerValue];
            if (isLoadAndShow == 1) {
                [_insertAdManager loadAndShow:parameters];
            }else {
                [_insertAdManager loadAd:parameters];
            }
        }else {
            [_splash loadAndShow:parameters];
        }
         
    } else if([method isEqualToString:@"showInterstitialAd"]){
        if(_insertAdManager){
            [_insertAdManager showAd];
        }
    } else if ([method isEqualToString:KEY_METHOD_BANNER_AD]) {
        
        NSNumber *width = parameters[KEY_BANNER_VIEW_WIDTH];
        NSNumber *height = parameters[KEY_BANNER_VIEW_HEIGHT];
      
        float configWidth = 0;
        if([width  isKindOfClass:NSNumber.class] && width.floatValue > 0){
            configWidth = width.floatValue;
        }
        float configHeight = 0;
        if([height  isKindOfClass:NSNumber.class] && height.floatValue > 0){
            configHeight = height.floatValue;
        }
        
        self.bannerView = [[YFAdsBannerView alloc]initWithFrame:CGRectMake(0,0,configWidth,configHeight) arguments:parameters eventSink:_flutterEventSink viewController:_flutterViewController];
       [[YFAdsSDKManager shareInstance].bannerViews setObject:self.bannerView forKey:parameters[KEY_UNIQUE_KEY]];
        
    } else if ([method isEqualToString:KEY_METHOD_REQUEST_REWARD_VIDEO_AD]) {
        _rewardVideoManager = [[YFAdsRewardVideoManager alloc] initWithParams:_flutterEventSink viewController:_flutterViewController result:result];
        if(parameters[@"loadAndShow"] && ![parameters[@"loadAndShow"] isKindOfClass:NSNull.class]) {
            NSInteger isLoadAndShow = [parameters[@"loadAndShow"] integerValue];
            if (isLoadAndShow == 1) {
                [_rewardVideoManager loadAndShow:parameters];
            }else {
                [_rewardVideoManager loadAd:parameters];
            }
        }else {
            [_rewardVideoManager loadAndShow:parameters];
        }

    } else if([method isEqualToString:@"showRewardVideoAd"]){
        if(_rewardVideoManager){
            [_rewardVideoManager showAd];
        }
    } else if ([method isEqualToString:KEY_METHOD_BANNER_PAGE_CLOSED]) {
        if(_bannerViewFactory){

            [_bannerViewFactory bannerViewPageFinished];

        }
    } else if ([method isEqualToString:KEY_METHOD_NATIVE_PAGE_CLOSED]) {
        if(_nativeViewFactory){
       
            [_nativeViewFactory nativeViewPageFinished];
//            if(parameters[KEY_UNIQUE_KEY]){
//                [[YFAdsSDKManager shareInstance].bannerViews removeObjectForKey:parameters[KEY_UNIQUE_KEY]];
//            }
        }
    } else if ([method isEqualToString:KEY_METHOD_VIDEO_PATH_PAGE_CLOSED]) {
        if(_videoPathViewFactory){
            [_videoPathViewFactory videoPathViewPageFinished];
//            if(parameters[KEY_UNIQUE_KEY]){
//                [[YFAdsSDKManager shareInstance].bannerViews removeObjectForKey:parameters[KEY_UNIQUE_KEY]];
//            }
        }
    }
}

- (FlutterError *_Nullable)onCancelWithArguments:(id _Nullable)arguments {
    _flutterEventSink = nil;
    return nil;
}

- (FlutterError *_Nullable)onListenWithArguments:(id _Nullable)arguments
                                       eventSink:
                                               (nonnull FlutterEventSink)events {
    _flutterEventSink = events;
    if(_bannerViewFactory){
        [_bannerViewFactory setFlutterEventSink:_flutterEventSink];
    }
    if(_nativeViewFactory){
        [_nativeViewFactory setFlutterEventSink:_flutterEventSink];
    }
    if(_videoPathViewFactory){
        [_videoPathViewFactory setFlutterEventSink:_flutterEventSink];
    }
    return nil;
}

@end
